// Lz4Decode:
#ifndef FLAT32
//   input   ds:si=inStream, es:di=outStream
//   output  outStream[], ds:si, es:di
	.code16
#define AX	%ax
#define BX	%bx
#define CX	%cx
#define SI	%si
#define DI	%di
#else
//   input   esi=inStream, edi=outStream
//   output  outStream[], ds:esi, es:edi
	.code32
#define AX	%eax
#define BX	%ebx
#define CX	%ecx
#define SI	%esi
#define DI	%edi
#endif
 
#define ARCHIVE_MAGICNUMBER 0x184C2102

//#define PARANOIA			// cover rare cases, optional

lz4main:
#ifdef PARANOIA
	cld
# if !defined(FLAT32) && !defined(FLAT16OUT)
	xorl	%cx, %cx
	call	lz4mov
# endif
#endif
	lodsl			// get chunkSize
#ifndef NO_LZ4_HEADER
	cmpl	$ARCHIVE_MAGICNUMBER, %eax
	je	lz4main
#endif
	orl	%eax, %eax	// end of file ?
	je	lz4quit
	xchgl	%eax, %edx
lz4chunk:			// uncompress chunk
	lodsb			// get token
	movb	%al, %bl
	shrb	$4, %al
	call	lz4len		// get literal length
	subl	%ecx, %edx	// count literal
#if !defined(FLAT32) && !defined(FLAT16OUT) && (!defined(FLAT16) || !defined(PARANOIA))
#define NeedLz4mov
	call	lz4mov		// copy literals
#else
	rep movsb
#endif
	subl	$1+2, %edx	// count token & string address
	jle	lz4main
	lodsw			// get string address
	xchg	AX, BX
	call	lz4len		// get string length
	add	$4, CX
#if !defined(FLAT32) && !defined(FLAT16OUT)
	pushw	%ds
	pushw	%si
	movw	%di, %si
	subw	%bx, %si
	movw	%es, %ax
	jnc	axok
	subb	$0x10, %ah
axok:
.macro norm	reg
	movw	%si, \reg
	andw	$0xF, %si
	shrw	$4, \reg
	addw	\reg, %ax
	movw	%ax, %ds
	movw	%di, \reg
	andw	$0xF, %di
	shrw	$4, \reg
	movw	%es, %ax
	addw	\reg, %ax
	movw	%ax, %es
.endm 
# if !defined (NeedLz4mov)
	norm	%bp
	rep movsb
# else
	call	lz4movStr	// copy string
# endif
	popw	%si
	popw	%ds
#else
	xchg	AX, SI
	mov	DI, SI
	sub	BX, SI
	rep movsb %es:(SI), %es:(DI)
	xchg	AX, SI
#endif
	jmp	lz4chunk

#if defined(NeedLz4mov)
# if defined(PARANOIA)
lz4movlp:
	xchgw	%ax, %cx
	movw	$0x3C00, %cx
	rep movsl
	xchgw	%ax, %cx
	sub	$0xF0, %ch
# endif
lz4mov:
	movw	%ds, %ax
lz4movStr:
	norm	%bp
# if defined(PARANOIA)
	cmp	$0xFF, %ch	// catch FFFX case
	jz	lz4movlp
# endif
	rep movsb
	ret
#endif

lz4len:				// get length in %ecx
	andl	$0xF, %eax
	movl	%eax, %ecx
	cmpb	$0xF, %al
	jne	lz4quit
lz4len2:
	lodsb
	decl	%edx		// remaining chunk size
	addw	%ax, %cx
	cmpb	$0xFF, %al
	je	lz4len2
lz4quit:
	ret
